(***************************************************************************
  PJB January 9, 1994, Internet mail to d91-pbr@nada.kth.se
  Copyright PJB 1994, All Rights Reserved.
  Free source, use at your own risk.
  If modified, please state so if you pass this around.

  This is a demonstration of the EDITLIST unit.  Three windows are
  created so you can try out the Window List dialog, and there is a tool
  menu with accompanying dialog.

  Define UseToyApp to inherit TToyApp rather than THelpApp. The
  differences are minor, User Screen swapping gets TVToys aware.

  Problems:
    The Window List searches the desktop for windows, but there is no
    way to find out which objects inserted into the desktop should
    actually be considered windows! The current algorithm adds
    everything that is selectable. You might have to edit DoAddWin to
    add only the windows you want.

    The maximum amount of tools is not enforced.
    You have to add your own tool macros...
    There is no validator for the Tool's command line.

***************************************************************************)
program ToolTest;
{$B-,X+}

  uses
    Dos,
    App, Dialogs, Drivers, Memory, Menus, MsgBox, Objects, Views,
    toyPrefs, HelpCtx,
    EditList, Video, toyApp, toyUtils, TVUtils;


  const
    cmTool       = 100;
    cmWindowList = 101;
    cmUserScreen = 102;

    cmInvokeTool = 1000;  { [1000..1019] are the default tool commands }
    { The number of commands to reserve for tools, should be less than
      the minimum number of lines on the screen! }
    MaxTools     = 1;


(***************************************************************************
***************************************************************************)

  type
    TToolTestApp =
     {$IFDEF UseToyApp}
      object (TToyApp)
     {$ELSE}
      object (THelpApp)
        DosVideoState  : VideoState;
     {$ENDIF}

        Tools      : TCollection;
        ToolMenu   : PMenu;

        constructor Init;
        destructor Done; virtual;
        procedure CreateToolMenu;
        procedure HandleEvent(var Event:TEvent); virtual;
        procedure InitMenubar; virtual;
        procedure InitStatusLine; virtual;
        procedure ResumeTV; virtual;
        procedure ShowUserScreen;
        procedure SuspendTV; virtual;
        procedure ToolDialog;
        procedure Transfer(TransferItem:Integer);
        procedure WindowList;
      end;


  (*******************************************************************
    Initialize
  *******************************************************************)
  constructor TToolTestApp.Init;
    var
      i : Integer;
      R : TRect;
      TempUserScreen : VideoState;
  begin
    CheckVideoType;
    TempUserScreen.Init;
    TempUserScreen.Save;
    TempUserScreen.SaveText;

    inherited Init;

    DosVideoState:=TempUserScreen;

    Tools.Init(10,10);

    { For demonstration purposes. 6 and 7 are ordinal values in a list where
      F6 is intentionally missing, so represent F7 and F8 }
    Tools.Insert(New(PToolListItem, InitData('~X~Copy help', 'xcopy', '/?', 6)));
    Include(toolUsedFKeys, 6);
    Tools.Insert(New(PToolListItem, InitData('~D~ialog Design', 'dlgdsn', '', 7)));
    Include(toolUsedFKeys, 7);

    Tools.Insert(New(PToolListItem, Init));  (* One empty always required *)
    CreateToolMenu;

    { To demonstrate the Window List }
    R.Assign(0,0,5,5);
    for i:=1 to 3 do
      if InsertWindow(New(PWindow, Init(R, 'Test '+Chr(i+48), i)))<>Nil then
        Desktop^.First^.Options:=Desktop^.First^.Options or ofTileable;
    Cascade;

    HelpFileName:='HELPTEST.HLP';
    ShowHelp(hcToolIntro);
  end;


  (*******************************************************************
    Restore Dos text
  *******************************************************************)
  destructor TToolTestApp.Done;
  begin
    inherited Done;
    DosVideoState.RestoreText;
  end;


  (*******************************************************************
    Create the Tool menu
  *******************************************************************)
  procedure TToolTestApp.CreateToolMenu;
    var
      i       : Integer;
      P       : PMenuItem;
  begin
    DisposeMenuItems(ToolMenu^.Items);
    P:=Nil;

    for i:=Tools.Count-2 downto 0 do
      with PToolListItem(Tools.At(i))^ do
        P:=NewItem(Title, HotKeyStr(HotKey),
          kbHotKey(kbShiftF1, HotKey), cmInvokeTool+i, hcNoContext, P);

    ToolMenu^.Items:=P;
    ToolMenu^.Default:=P;
  end;


  (*******************************************************************
    Events
  *******************************************************************)
  procedure TToolTestApp.HandleEvent;
  begin
    inherited HandleEvent(Event);
    if Event.What=evCommand then
    begin
      case Event.Command of
        cmCloseAll:             CloseAll;
        cmInvokeTool..cmInvokeTool+MaxTools-1:
                                Transfer(Event.Command-cmInvokeTool);
        cmTool:                 ToolDialog;
        cmUserScreen:           ShowUserScreen;
        cmWindowList:           WindowList;
        else
          Exit;
      end;
      ClearEvent(Event);
    end;
  end;


  (*******************************************************************
    Demonstration Menu bar
  *******************************************************************)
  procedure TToolTestApp.InitMenubar;
    var
      R : TRect;
  begin
    GetExtent(R);
    R.B.Y:=R.A.Y+1;
    MenuBar:=New(PMenuBar, Init(R, NewMenu(
      NewSubMenu('~F~ile', hcNoContext, NewMenu(
        NewItem('~D~OS shell', '', kbNoKey, cmDosShell, hcDosShell,
        NewItem('E~x~it', 'Alt+X', kbAltX, cmQuit, hcExit,
        nil))),
      NewSubMenu('~T~ools', hcNoContext,
        StorePointer(ToolMenu, NewMenu(
        nil)),
      NewSubMenu('~O~ptions', hcNoContext, NewMenu(
        NewItem('~T~ools...', '', kbNoKey, cmTool, hcNoContext,
        nil)),
      NewSubMenu('~W~indow', hcNoContext, NewMenu(
        StdWindowMenuItems(
        NewLine(
        NewItem('~L~ist...', 'Alt+0', kbAlt0, cmWindowList, hcNoContext,
        NewLine(
        NewItem('~U~ser screen...', 'Alt+F5', kbAltF5, cmUserScreen, hcNoContext,
        nil)))))),
      nil)))))));
  end;


  (*******************************************************************
    Demonstration status line. Uses some DOS 6 Help keys
  *******************************************************************)
  procedure TToolTestApp.InitStatusLine;
    var
      R: TRect;
  begin
    GetExtent(R);
    R.A.Y := R.B.Y - 1;
    New(StatusLine, Init(R,
      StdStatusHelp(                  (* It's in TOYPREFS.PAS *)
      NewStatusDef(0, $FFFF,
        StdStatusKeys(
        NewStatusKey('~F1~ Help',    kbF1,   cmHelp,
        NewStatusKey('~Alt+X~ Exit', kbAltX, cmQuit,
      Nil))),
    Nil))));
  end;


  (*******************************************************************
    Save user screen after Dos shell
  *******************************************************************)
  procedure TToolTestApp.ResumeTV;
  begin
    if IsProbablyTextMode then
      DosVideoState.SaveText;
    inherited ResumeTV;
  end;


  (*******************************************************************
    Show the user screen, ie the DOS screen
  *******************************************************************)
  procedure TToolTestApp.ShowUserScreen;
  begin
    SuspendTV;

    (* Wait for a keypress, don't use TV but BIOS Get Key *)
    asm
      mov  ah,0
      int  16h
    end;

    ResumeTV;
    Redraw;
  end;


  (*******************************************************************
    Restore user screen before Dos shell
  *******************************************************************)
  procedure TToolTestApp.SuspendTV;
  begin
    inherited SuspendTV;
    if IsProbablyTextMode then
      DosVideoState.RestoreText;
  end;


  (*******************************************************************
    Edit tools
  *******************************************************************)
  procedure TToolTestApp.ToolDialog;
    var
      DataRec : TListBoxRec;

    function CopyObject(P:PObject):PObject;
      var
        NewObject : PObject;
    begin
      GetMem(NewObject, SizeOf(P^));
      Move(P^, NewObject^, SizeOf(P^));
      CopyObject:=NewObject;
    end;

    procedure CopyCollection(P:PObject); far;
    begin
      DataRec.List^.Insert(CopyObject(P));
    end;

  begin
    New(DataRec.List, Init(Tools.Count, 5));
    Tools.ForEach(@CopyCollection);

    DataRec.Selection:=0;

    if ExecuteDialog(MakeStandardToolDialog(MaxTools), @DataRec)=cmOK then
    begin
      Tools.Done;
      Tools:=DataRec.List^;
      CreateToolMenu;
    end
    else
      Dispose(DataRec.List, Done);
  end;


  (*******************************************************************
    Transfer
  *******************************************************************)
  procedure TToolTestApp.Transfer(TransferItem:Integer);
    var
      Swap : Boolean;

    (*******************************************************************
      Look for macros on the command line
    *******************************************************************)
    function CheckCommandLine(const Params:String):String;
      var
        i      : Integer;
        Quit   : Boolean;
        Result : String;

      (*******************************************************************
        Copy character from tool's command line to program's parameter str
      *******************************************************************)
      procedure CopyChar;
      begin
        Inc(Result[0]);
        Result[Length(Result)]:=Params[i];
        Inc(i);
      end;

      (*******************************************************************
        Try to match cureent position with uppercase $macro
      *******************************************************************)
      function Match(const Macro:String):Boolean;
        var
          k : Integer;
      begin
        Match:=False;
        if i+Length(Macro)<=Length(Params) then
          for k:=1 to Length(Macro) do
            if Upcase(Params[i+k])<>Macro[k] then
              Exit;
        Inc(i, Length(Macro)+1);             (* Skip the $ too *)
        Match:=True;
      end;

    begin
      i:=1;
      Result:='';

      while i<=Length(Params) do
        case Params[i] of
          '$':                               (* Macro designator *)
            if (i>1) and (Params[i-1]<>' ') then
              CopyChar
            else
            begin
              if Match('NOSWAP') then
                Swap:=False;
            end;
          else
            CopyChar;
        end;

      CheckCommandLine:=Result;
    end;

    var
      Command    : String;
      Parameters : String;

  begin
    Swap:=True;
    with PToolListItem(Tools.At(TransferItem))^ do
    begin
      if not Swap then
        Notice('Transfer', ^M^M^C'Running '+Title);

      { Search the path for the program }
      Command:=Path;
      if Pos('.', Path)=0 then
        Command:=Path+'.COM;'+Path+'.EXE';

      Command:=FFindFile(Command, '.;'+GetEnv('PATH'));

      if Command='' then
        MessageBox(^C'Could not find tool'^M^C+GetText, Nil, mfError+mfOKButton)
      else
      begin
        Parameters:=CheckCommandLine(CommandLine);

        if Swap then
          SuspendTV
        else
        begin
          DoneSysError;
          DoneEvents;
          DoneDosMem;
        end;

        SwapVectors;
        Exec(Command, Parameters);
        SwapVectors;

        if Swap then
          ResumeTV
        else
        begin
          InitDosMem;
          InitEvents;
          InitSysError;
          Redraw;
        end;
      end;

      if not Swap then
        NoNotice;
    end;
  end;


  (*******************************************************************
    Window List dialog
  *******************************************************************)
  procedure TToolTestApp.WindowList;
    var
      WinList : TCollection;
      DataRec : TListBoxRec;
      P       : PDialog;

    procedure DoAddWin(P:PWindow); far;
    begin
      if P^.Options and ofSelectable<>0 then
        WinList.Insert(New(PWindowListItem, Init(P)));
    end;

  begin
    WinList.Init(10,10);
    DataRec.List:=@WinList;
    DataRec.Selection:=0;

    Desktop^.ForEach(@DoAddWin);

    (*******************************************************************
      Can't use ExecView since it restores the last selected window
      which might have been deleted...
    *******************************************************************)
    P:=MakeStandardWindowListDialog;
    if ValidView(P)<>Nil then
    begin
      P^.SetData(DataRec);
      P^.SetState(sfModal, True);
      if InsertWindow(P)<>Nil then
      begin
        if (P^.Execute=cmOK) and (WinList.Count>0) then
        begin
          P^.GetData(DataRec);
          PWindowListItem(WinList.At(DataRec.Selection))^.Window^.Select;
        end;
        Dispose(P, Done);
      end;
    end;

    Winlist.Done;
  end;


    (*******************************************************************
    *******************************************************************)

  var
    Main : TToolTestApp;

begin
  Main.Init;
  Main.Run;
  Main.Done;
end.