(***************************************************************************
  Utils unit
  Odd stuff
  PJB November 3, 1993, Internet mail to d91-pbr@nada.kth.se
  Copyright 1993, All Rights Reserved
  Free source, use at your own risk.
  If modified, please state so if you pass this around.

  These are non-TV specific helpers with no error checking

***************************************************************************)
unit toyUtils;

{$B-,X+}

interface

  uses
    Dos;


  (*******************************************************************
    Dos 3.x+ SHARE constants for concurrent usage
    The RTM keeps the Exe file open, so opening the Exe to do your
    own processing with SHARE loaded requires stDenyNone
  *******************************************************************)
  const
    stCompatibilityMode = $0000;
    stDenyReadWrite     = $0010;
    stDenyWrite         = $0020;
    stDenyRead          = $0030;
    stDenyNone          = $0040;


  function  AddBackslash(const s:String):String;
  function  DefaultExtension(const FileName, Ext:String; Force:Boolean):String;

  procedure FlushSmartDrive;

  function  FFindFile(const FileList:String; const DirList:String):PathStr;
  procedure FirstInListThat(const List:String; Func:Pointer);
  procedure ForEachInList(const List:String; Proc:Pointer);

  function  HexChar(i:Integer):Char;
  function  HexStr(w:Word):String;
  function  HexStrValue(const s:String):Word;
  function  HexValue(c:Char):Byte;

  { Used to fetch the caller's stack frame, from BP's OUTLINE unit }
  function  CallerFrame:Word;
    inline ($8B/$46/0);  { MOV AX,[BP] }
  { For calling local procedures, see FFindFile }
  function  SecondCallerFrame:Word;
    inline ($8B/$5E/0/$36/$8B/7);  { MOV BX,[BP] MOV AX,SS:[BX] }

  function  Min(a,b:Integer):Integer;
  function  Max(a,b:Integer):Integer;
  function  MemComp(const m1, m2; Len:Integer):Boolean;
  function  ToStr(l:Longint):String;

  const
    HexChars : array [0..15] of char = '0123456789ABCDEF';


(***************************************************************************
***************************************************************************)
implementation


  (*******************************************************************
    Make sure we can append a file name
  *******************************************************************)
  function AddBackslash(const s:String):String;
  begin
    if (s<>'') and not (s[Length(s)] in ['\',':']) then
      AddBackslash:=s+'\'
    else
      AddBackslash:=s;
  end;


  (*******************************************************************
    Add extension if necessary
  *******************************************************************)
  function  DefaultExtension(const FileName, Ext:String; Force:Boolean):String;
    var
      P: PathStr;
      D: DirStr;
      N: NameStr;
      E: ExtStr;
  begin
    FSplit(FileName, D, N, E);
    if (E = '') or Force then
      E:=Ext;
    DefaultExtension:=D+N+E;
  end;


  (*******************************************************************
    Act on a semicolon separated list of items
  *******************************************************************)
  procedure TraverseList(const List:String; Proc:Pointer; ForEach:Boolean);
    type
      FEIListProc = function (const S:String; Frame:Word):Boolean;
    var
      i, j : Integer;
  begin
    i:=1;
    for j:=i to Length(List)+1 do
      if (j>Length(List)) or (List[j]=';') then
      begin
        if j>i then
        begin
     { We have to use the second local frame since we were called indirectly }
          if FEIListProc(Proc)(Copy(List, i,  j-i), SecondCallerFrame) and
               not ForEach then
            Exit;                   { Exit if FirstThat true }
        end;
        i:=j+1;
      end;
  end;


  (*******************************************************************
    Call Func with every item in the semicolon separated list
    until Func returns True. Func must be declared as
         function (const S:String):Boolean; far;
  *******************************************************************)
  procedure FirstInListThat(const List:String; Func:Pointer);
  begin
    TraverseList(List, Func, False);
  end;


  (*******************************************************************
    Call Proc for every item in the semicolon separated list
    Proc must be declared as   procedure (const S:String); far;
  *******************************************************************)
  procedure ForEachInList(const List:String; Proc:Pointer);
  begin
    TraverseList(List, Proc, True);
  end;


  (*******************************************************************
    FFindFile searches the DirList directories and returns the
    first directory in which any of the files listed in the
    FileList exists. The list items must separated by semi-colons.
    The FileList can contain wild cards. Lists are traversed from
    left to right.

    This procedure is a more powerful version of FSearch.

    Example:
      FFindFile('*.COM;*.EXE', GetEnv('PATH'))
      Returns the first executable file found in the path
  *******************************************************************)
  function FFindFile(const FileList:String; const DirList:String):PathStr;
    function SearchDir(const Path:String):Boolean; far;
      function FindFile(const S:String):Boolean; far;
        var
          SR : SearchRec;
      begin
        FindFirst(AddBackSlash(Path)+S, AnyFile-Directory, SR);
        if DosError=0 then
          FFindFile:=AddBackSlash(Path)+SR.Name;
        FindFile:=DosError=0;
      end;

    begin
      FirstInListThat(FileList, @FindFile);
      SearchDir:=DosError=0;
    end;

  begin
    FFindFile:='';
    FirstInListThat(DirList, @SearchDir);
  end;

  (*******************************************************************
    Flush the SmartDrive disk cache, same as smartdrv /c
  *******************************************************************)
  procedure FlushSmartDrive; assembler;
  asm
      mov  ax,4A10h
      mov  bx,1
      int  2Fh
  end;


  (*******************************************************************
    Convert 0-15 to hex (0-F)
  *******************************************************************)
  function HexChar;
  begin
    HexChar:=HexChars[i and 15];
  end;


  (*******************************************************************
    Convert word to four hex chars, zero extended
  *******************************************************************)
  function HexStr;
  begin
    HexStr[0]:=Chr(4);
    HexStr[1]:=HexChars[Hi(w) shr 4];
    HexStr[2]:=HexChars[Hi(w) and $F];
    HexStr[3]:=HexChars[Lo(w) shr 4];
    HexStr[4]:=HexChars[Lo(w) and $F];
  end;


  (*******************************************************************
    Convert hex string to word
  *******************************************************************)
  function HexStrValue;
    var
      sum, i : Word;
  begin
    sum:=0;
    for i:=1 to Length(s) do
      sum:=16*sum+HexValue(s[i]);
    HexStrValue:=sum;
  end;


  (*******************************************************************
    Convert hex char (0-F) to byte (0-15)
  *******************************************************************)
  function HexValue;
  begin
    c:=Upcase(c);
    if c>'9' then
      Dec(Byte(c), 7);
    HexValue:=Ord(c)-Ord('0');
  end;


  (*******************************************************************
    Min and Max
  *******************************************************************)
  function Min;
  begin
    if a<b then Min:=a else Min:=b;
  end;

  function Max;
  begin
    if a>b then Max:=a else Max:=b;
  end;


  (*******************************************************************
    Compare two blocks of memory, returns True if equal
  *******************************************************************)
  function MemComp; assembler;
  asm
      push ds
      les  di,m1
      lds  si,m2
      mov  cx,Len
      cld
      rep  cmpsb
      mov  al,1
      je   @Fin
      dec  al
    @Fin:
      pop  ds
  end;


  (*******************************************************************
    Function version of Str(), incurs speed overhead
  *******************************************************************)
  function ToStr;
    var
      s : String[15];
  begin
    Str(l,s);
    ToStr:=s;
  end;


end.
